Add a Force option to raw_fd_ostream to specify whether opening
an existing file is considered an error. Convert several tools
to use raw_fd_ostream instead of std::ostream, and to use this
new option instead of doing a manual check.


git-svn-id: https://llvm.org/svn/llvm-project/llvm/trunk@75801 91177308-0d34-0410-b5e6-96231b3b80d8
diff --git a/tools/llvm-as/llvm-as.cpp b/tools/llvm-as/llvm-as.cpp
index eccabd5..1d572f1 100644
--- a/tools/llvm-as/llvm-as.cpp
+++ b/tools/llvm-as/llvm-as.cpp
@@ -28,8 +28,6 @@
 #include "llvm/Support/SystemUtils.h"
 #include "llvm/Support/raw_ostream.h"
 #include "llvm/System/Signals.h"
-#include <fstream>
-#include <iostream>
 #include <memory>
 using namespace llvm;
 
@@ -62,7 +60,7 @@
   cl::ParseCommandLineOptions(argc, argv, "llvm .ll -> .bc assembler\n");
 
   int exitCode = 0;
-  std::ostream *Out = 0;
+  raw_ostream *Out = 0;
   try {
     // Parse the file now...
     SMDiagnostic Err;
@@ -86,23 +84,24 @@
 
     if (OutputFilename != "") {   // Specified an output filename?
       if (OutputFilename != "-") {  // Not stdout?
-        if (!Force && std::ifstream(OutputFilename.c_str())) {
-          // If force is not specified, make sure not to overwrite a file!
-          cerr << argv[0] << ": error opening '" << OutputFilename
-               << "': file exists!\n"
-               << "Use -f command line argument to force output\n";
+        std::string ErrorInfo;
+        Out = new raw_fd_ostream(OutputFilename.c_str(), /*Binary=*/true,
+                                 Force, ErrorInfo);
+        if (!ErrorInfo.empty()) {
+          errs() << ErrorInfo << '\n';
+          if (!Force)
+            errs() << "Use -f command line argument to force output\n";
+          delete Out;
           return 1;
         }
-        Out = new std::ofstream(OutputFilename.c_str(), std::ios::out |
-                                std::ios::trunc | std::ios::binary);
       } else {                      // Specified stdout
-        // FIXME: cout is not binary!
-        Out = &std::cout;
+        // FIXME: outs() is not binary!
+        Out = &outs();
       }
     } else {
       if (InputFilename == "-") {
         OutputFilename = "-";
-        Out = &std::cout;
+        Out = &outs();
       } else {
         std::string IFN = InputFilename;
         int Len = IFN.length();
@@ -114,27 +113,22 @@
         }
         OutputFilename += ".bc";
 
-        if (!Force && std::ifstream(OutputFilename.c_str())) {
-          // If force is not specified, make sure not to overwrite a file!
-          cerr << argv[0] << ": error opening '" << OutputFilename
-               << "': file exists!\n"
-               << "Use -f command line argument to force output\n";
+        std::string ErrorInfo;
+        Out = new raw_fd_ostream(OutputFilename.c_str(), /*Binary=*/true,
+                                 Force, ErrorInfo);
+        if (!ErrorInfo.empty()) {
+          errs() << ErrorInfo << '\n';
+          if (!Force)
+            errs() << "Use -f command line argument to force output\n";
+          delete Out;
           return 1;
         }
-
-        Out = new std::ofstream(OutputFilename.c_str(), std::ios::out |
-                                std::ios::trunc | std::ios::binary);
         // Make sure that the Out file gets unlinked from the disk if we get a
         // SIGINT
         sys::RemoveFileOnSignal(sys::Path(OutputFilename));
       }
     }
 
-    if (!Out->good()) {
-      cerr << argv[0] << ": error opening " << OutputFilename << "!\n";
-      return 1;
-    }
-
     if (!DisableOutput)
       if (Force || !CheckBitcodeOutputToConsole(Out,true))
         WriteBitcodeToFile(M.get(), *Out);
@@ -146,7 +140,7 @@
     exitCode = 1;
   }
 
-  if (Out != &std::cout) delete Out;
+  if (Out != &outs()) delete Out;
   return exitCode;
 }